/*====================================================================================
   This file is part of the TechTools Plugin Development Kit. 
   Copyright (c) 2011 by TechTools  

   You may modify this file and/or create derived works to develop plugins 
   for use with TechTools's DigiView software and capture files obtained from
   TechTools' hardware. Any other use is prohibited.

            email: support@tech-tools.com,  sales@tech-tools.com 
		web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

#include "../plugin.h"

// ASYNC specific event flags
#define BRKEVENT   0x80
#define ENDEVENT   0x40
#define PERROR     0x20
#define FERROR     0x10
#define ADDRBYTE   0x02
#define DATAEVENT  0x01

static bool escapeflag;
static bool useescape;
static unsigned char escapechar;
static int64 escapetime;
static unsigned char SOFchar;
static bool ShowFieldIdle;

void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed
void SetInitItem(unsigned char ID, unsigned char subID, int value) {} // don't care about globals
void EndOfData() {}    // no end-of-parsing finalization needed

enum opts {SHOWIDLE,SOFCHAR,ESCCHAR,USEESC,OPTSEND};
enum field_formats {FFsof,FFdata,FFperr,FFferr,FFbrk,FFEND};

void GetStrList(int ID, vector<string> &strl)
{	
	switch (ID) 
	{  // 5 (lookuptables) returns empty list
	case 0: strl.push_back("SOF & Escape ASYNC example V0.1 BETA"); break;
	case 1: 
		strl.assign(OPTSEND,"");
		strl[SHOWIDLE] = "Show Field Idle,checkbox,false";
		strl[SOFCHAR] =  "Framing Character,edit,1";
		strl[ESCCHAR] =  "Escape Character,edit,27";
		strl[USEESC]  =  "Use Escape Character,checkbox,false";
		break;
	case 2: // field formats 
		strl.assign(FFEND,"");
		strl[FFsof]  = "SOF,GREEN,BLACK,{7:0}"; 
		strl[FFdata] = "DATA,,,{7:0}";
		strl[FFperr] = "ParityError,RED,WHITE,P";
		strl[FFferr] = "FrameError,RED,WHITE,F";
		strl[FFbrk]  = "BREAK,YELLOW,BLACK,BREAK";
		break; 
	case 3:  strl.push_back("ASYNC"); break;      // use ASYNC pre-parser
	case 4:  strl.push_back("1");	break;        // require framework version 1 or above
	}
}


void SetCfgItem(unsigned char ID, unsigned char subID, int value) 
{ 
	switch (ID)
	{
	case SHOWIDLE:
		if (subID == 0) ShowFieldIdle = (value != 0);
		break;
	case SOFCHAR:
		if (subID == 0) SOFchar = (unsigned char)(value);
		break;
	case ESCCHAR:
		if (subID == 0) escapechar = (unsigned char)(value);
		break;
	case USEESC:
		if (subID == 0) useescape = (value != 0);
		break;
	}
}


void StartOfData() 
{ 	
	escapeflag = false;
}


void Parse(int64 timestamp, Data64 data)
{
	if (data.bytes[6] & DATAEVENT)
	{   // handle SOF chars and escapes
		if (escapeflag) // last char was an escape char...send this char as a normal character
		{ 
			StartField(escapetime,data,FFdata);  // output it with DATA field format
			escapeflag = false;  // cancel escape mode
		}

		else if (useescape & (data.bytes[0] == escapechar)) 
		{   escapetime = timestamp; // remember when this occured
			escapeflag = true; // set escape mode, do NOT print this character
		}

		else if (data.bytes[0] == SOFchar) 
		{  // start a new frame with this character
			StartFrame(timestamp,data,FFsof);  // output it with SOF field format
		}

		else 
		{  // just another regular field
			StartField(timestamp,data,FFdata);  // output it with DATA field format
		}

	}  // data event

	// note use of 'else'. Multiple event flags can be set along with the data event to give us
	// early notice of any errors. However, we will receive additional events later (with individual
	// timestamps) to handle the errors. We choose to handle them individually.
	else if (data.bytes[6] & PERROR)  // handle parity error events
		StartField(timestamp,data,FFperr);  // output it with Parity error field format

	else if (data.bytes[6] & FERROR)  // handle framing error events
		StartField(timestamp,data,FFferr);  // output it with Frame error field format

	else if (data.bytes[6] & BRKEVENT)  // handle BREAK events
		StartField(timestamp,data,FFbrk);  // output it with BREAK field format 
    
	else if (ShowFieldIdle && (data.bytes[6] & ENDEVENT))  // handle end-of-char events
		EndField(timestamp);  // start IDLE (doesn't use formats)

}