/*====================================================================================
   This file is part of the TechTools Plugin Development Kit. 
   Copyright (c) 2011 by TechTools  

   You may modify this file and/or create derived works to develop plugins 
   for use with TechTools's DigiView software and capture files obtained from
   TechTools' hardware. Any other use is prohibited.

            email: support@tech-tools.com,  sales@tech-tools.com 
		web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

#include "../plugin.h"

// NOTE: this is a theoretical exercise ( a nice way of saying it has not been tested against real data)

// This particular device packs 3 pieces of information into a single 16 bit field.  SPI requires a fixed
// field length so we must accept all 3 pieces of information as a single field with a single timestamp
// if we base the plugin on the SPI pre-parser.  We can still extract the 3 pieces of data with data slices
// but the data has to be printed in a single field.  We use lookup tables to decode the commands.  This 
// might not be as pretty as separate fields but is easy and fully functional.  You can still search on 
// each part (specific channel,command and/or data).

// This could have been done as a FULL parser or it could have been based on the SYNC pre-parser.  Either
// of these options would have given us more timestamp information we could have used to break up information
// into separate fields. We chose to use the SPI pre-parser for this one to show use of slices and lookups 
// and to demonstrate how easy it can be to layer simple decoding on top of a built-in parser.

// This device has a simmple data format:
// top 2 bits are channel select
// next 2 bits are the command
// next 8 bits are the data
// last 4 bits are padding

// If this were a real plugin, we would include notes in the help file to remind the user to set
// the Field Length to 16 and to disable the MOSI data.

// SPI specific event flags
#define DATAFLAG    0x80
#define TRUNCFLAG   0x40
#define SSENFLAG    0x20
#define SSDISFLAG   0x10
#define ENDFLAG     0x08

void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed
void SetInitItem(unsigned char ID, unsigned char subID, int value) {} // don't care about globals
void SetCfgItem(unsigned char ID, unsigned char subID, int value) { } // no cfg items to set
void StartOfData() {}  // no init neeeded
void EndOfData() {}    // no end-of-parsing finalization needed

enum field_formats {FFDATA,FFTRUNC,FFEND};

void GetStrList(int ID, vector<string> &strl)
{	
	switch (ID) 
	{  
	case 0: strl.push_back("SPI-DAC8045S085 Example V0.1 BETA"); break;
//	case 1: // no plugin cfg items
	case 2: // field formats 
		strl.assign(FFEND,"");
		strl[FFDATA]  = "Data,,,Ch{0%15:14} / {1%13:12} / {11:4}"; 
		strl[FFTRUNC] = "Truncated,80FFFF,BLACK,Ch{0%15:14} / {1%13:12} / {11:4}"; 
		break;
	case 3:  strl.push_back("SPI"); break;      // use SPI pre-parser
	case 4:  strl.push_back("1");	break;      // require framework version 1 or above
	case 5:
		strl.push_back("0,0,A");
		strl.push_back("0,1,B");
		strl.push_back("0,2,C");
		strl.push_back("0,3,D");
		strl.push_back("1,0,WT Ch");
		strl.push_back("1,1,Update Ch");
		strl.push_back("1,2,Update ALL");
		strl.push_back("1,3,Power-Down");
		break;
	}
}


void Parse(int64 timestamp, Data64 data)
{
	if (data.bytes[6] & DATAFLAG)
	{
		if (data.bytes[6] & TRUNCFLAG)
		    StartField(timestamp,data,FFTRUNC); 
		else
			StartField(timestamp,data,FFDATA); 
	}

	if (data.bytes[6] & ENDFLAG)
		EndField(timestamp);
}

/*  NOTE:
One simple modification to this would be to look at the upper 4 bits as a single command
and evaluate it in the Parse() routine.  Then use a single lookup table to select the combined
data.  It could look like  'Wt CHA W/O update: 55'

*/